/*
 * sunny_platform_widgets
 * Copyright (c) 2018 Lance Johnstone. All rights reserved.
 * See LICENSE for distribution and usage details.
 */

import 'package:flutter/cupertino.dart' show CupertinoTheme, CupertinoThemeData;
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart' show Theme, ThemeData, Colors;
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';
import 'package:sunny_platform_widgets/sunny_platform_widgets.dart';

/// Extends on [TargetPlatform] to include web
enum PlatformTarget {
  /// Android: <https://www.android.com/>
  android,

  /// Fuchsia: <https://fuchsia.googlesource.com/>
  fuchsia,

  /// iOS: <https://www.apple.com/ios/>
  iOS,

  /// Linux: <https://www.linux.org>
  linux,

  /// macOS: <https://www.apple.com/macos>
  macOS,

  /// Windows: <https://www.windows.com>
  windows,

  web,
}

PlatformStyle _platformStyle(BuildContext context) {
  final platform = PlatformProvider.of(context)?.platform;

  final platformStyle = PlatformProvider.of(context)?.settings.platformStyle;

  if (platform == null && kIsWeb) {
    return platformStyle?.web ?? PlatformStyle.Material;
  }

  switch (platform ?? Theme.of(context).platform) {
    case TargetPlatform.android:
      return platformStyle?.android ?? PlatformStyle.Material;
    case TargetPlatform.fuchsia:
      return platformStyle?.fuchsia ?? PlatformStyle.Material;
    case TargetPlatform.iOS:
      return platformStyle?.ios ?? PlatformStyle.Cupertino;
    case TargetPlatform.linux:
      return platformStyle?.linux ?? PlatformStyle.Material;
    case TargetPlatform.macOS:
      return platformStyle?.macos ?? PlatformStyle.Cupertino;
    case TargetPlatform.windows:
      return platformStyle?.windows ?? PlatformStyle.Material;
  }
}

bool isMaterial(BuildContext context) {
  return _platformStyle(context) == PlatformStyle.Material;
}

bool isCupertino(BuildContext context) {
  return _platformStyle(context) == PlatformStyle.Cupertino;
}

T platformThemeData<T>(
  BuildContext context, {
  required T Function(ThemeData theme) material,
  required T Function(CupertinoThemeData theme) cupertino,
}) {
  return isMaterial(context)
      ? material(Theme.of(context))
      : cupertino(CupertinoTheme.of(context));
}

PlatformTarget _webPlatformTarget = PlatformTarget.web;
set webPlatformTarget(PlatformTarget target) {
  _webPlatformTarget = target;
}

PlatformTarget platform(BuildContext context) {
  if (kIsWeb) {
    return _webPlatformTarget;
  }

  switch (Theme.of(context).platform) {
    case TargetPlatform.android:
      return PlatformTarget.android;
    case TargetPlatform.fuchsia:
      return PlatformTarget.fuchsia;
    case TargetPlatform.iOS:
      return PlatformTarget.iOS;
    case TargetPlatform.linux:
      return PlatformTarget.linux;
    case TargetPlatform.macOS:
      return PlatformTarget.macOS;
    case TargetPlatform.windows:
      return PlatformTarget.windows;
  }
}

typedef Future<T?> ShowPlatformDialog<T>({
  required BuildContext context,
  required WidgetBuilder builder,
  bool? barrierDismissible,
  RouteSettings? routeSettings,
  bool useRootNavigator,
  bool materialUseSafeArea,
  Color? materialBarrierColor,
  String? barrierLabel,
  Constraints? constraints,
  extraOptions,
});

typedef ShowPlatformDialog<T> ShowPlatformDialogFactory<T>();

Future<T?> showPlatformDialog<T>({
  required BuildContext context,
  required WidgetBuilder builder,
  bool? barrierDismissible,
  RouteSettings? routeSettings,
  bool useRootNavigator = true,
  bool materialUseSafeArea = true,
  Color? materialBarrierColor = Colors.black54,
  String? barrierLabel,
  extraOptions,
}) {
  PlatformOverrides? state =
      context.findAncestorWidgetOfExactType<PlatformOverrides>();
  final opener = state?.data.findShowPlatformDialog<T>(
        context,
        extraOptions,
      ) ??
      defaultShowPlatformDialog<T>();

  return opener(
      context: context,
      builder: builder,
      barrierDismissible: barrierDismissible,
      routeSettings: routeSettings,
      materialUseSafeArea: materialUseSafeArea,
      materialBarrierColor: materialBarrierColor,
      barrierLabel: barrierLabel);
}

typedef Future<T?> ShowPlatformModalSheet<T>({
  required BuildContext context,
  required WidgetBuilder builder,
  MaterialModalSheetData? material,
  CupertinoModalSheetData? cupertino,
});

typedef Future<DateTime?> ShowDatePicker({
  required BuildContext context,
  required DateTime initialDate,
  required DateTime firstDate,
  required DateTime lastDate,
  PlatformBuilder<MaterialDatePickerData>? material,
  PlatformBuilder<CupertinoDatePickerData>? cupertino,
  DatePickerContentBuilder? cupertinoContentBuilder,
  dynamic extraOptions,
});

/// Displays either the showModalBottomSheet for material
/// or showCupertinoModalPopup for cupertino
Future<T?> showPlatformModalSheet<T>({
  required BuildContext context,
  required WidgetBuilder builder,
  MaterialModalSheetData? material,
  CupertinoModalSheetData? cupertino,
  extraOptions,
}) {
  PlatformOverrides? state =
      context.findAncestorWidgetOfExactType<PlatformOverrides>();
  final opener = state?.data.findShowPlatformModalSheet<T>(
        context,
        extraOptions,
      ) ??
      defaultShowPlatformModalSheet<T>();
  return opener(
    context: context,
    builder: builder,
    material: material,
    cupertino: cupertino,
  );
}
